<?php
if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

/**
 * WorthstockDataMapper è la classe che dato un prodotto Woocommerce si occupa di generare il JSON formattato correttamente per le richieste API da inviare a WORTHSTOCK
 * 
 * Il JSON varia parzialmente per il tipo di richiesta (Update o Insert, UpdatePrice, UpdateStock) e produce una relazione 1 prodotto WooCommerce a molti prodotti WORTHSTOCK + Variazioni in alcuni casi (dipende dalla presenza di Attributi default o Attributi custom)
 */
class WorthstockDataMapper extends WorthstockBase {

    private $woo_product;
    private $woo_product_id;
    private $woo_product_variations_elaborated;
    private $operation_type;
    private $default_product;
    private $default_variants;
    private $min_unmanaged_stock;
    public $default_attributes;

    public function __construct()
    {
        $this->woo_product = $this->woo_product_id = $this->operation_type = null;
        $this->woo_product_variations_elaborated = array();
        $this->default_product = worthstock_get_product_defaults();
        $this->default_variants = worthstock_faker_get_sample_product_variants();
        $this->default_attributes = array("Size", "Color", "Material", "NA");
        $this->min_unmanaged_stock = 10;
    }

    /**
     * there just one woocommerce product for each mapping, so saving the value inside the class
     */
    protected function instantiateProduct($product_id, $operation_type) {        
        $this->woo_product = $this->woo_product_id = $this->operation_type = null;
        $product = wc_get_product( $product_id );
        if (!$product_id || !$product) return false;
        $this->woo_product_id = $product_id;
        $this->operation_type = $operation_type;
        return $this->woo_product = $product;
    }

    protected function getDefaultAttributesMap() {
        $allowed_attributes_map = array(
            "size" => array("size","taglia","taille","size","sizes","taglie","dimensione","dimensioni","misura","misure"),
            "color" => array("color","colour","colore","couleur", "colors", "colours","colori", "tonalità", "tinta","tinte", "tintura", "tinture"),
            "material" => array("material","materiale","matériel","materials","materiale","materiali")
        );
        return $allowed_attributes_map;
    }

    /**
     * map attribute name to corresponding WS default attribute name (es. misura->size)
     */
    public function translateDefaultAttribute($attribute_name) {
        $attributes_map = $this->getDefaultAttributesMap();
        foreach ($attributes_map as $default_attr => $versions) {
            if ( in_array(mb_strtolower($attribute_name), $versions) ) {
                return ucfirst($default_attr);
            }
        }
        return false;
    }


    /**
     * takes a woocommerce product_id and return data in worthstock format
     */
    public function convertProductToWorthstock($product_id, $action_type="insert")
    {
        /* $product = wc_get_product( $product_id );
        if (!$product_id || !$product) return false;
        */
        if ( ($product = $this->instantiateProduct($product_id, $action_type)) === false ) {
            return false;
        }
        // compatibilità con precedente parametro
        $filter_type = ($action_type == 'update' || $action_type == 'insert') ? 'full' : $action_type;
        $data = [];
        if ($this->default_product) {
            $data = $this->default_product;
            unset($data["IdProduct"]);
        }
        //$data = $this->getProductWorthstockMetaFields($product_id);
        if ($id_worthstock = $this->readWorthstockCustomParamValue($product_id, 'IdProduct')) {
            $data["IdProduct"] = (is_array($id_worthstock)) ? $id_worthstock[0] : $id_worthstock;
        }        

        //$worthstock_custom_meta = $this->main()->sync->getProductWorthstockMetaFields($product_id);
        $worthstock_custom_meta = $this->readWorthstockCustomAllValues($product_id);
        
        // se non viene passato un brandname imposto al site name (rimosso come da richiesta di Pietro)
        // if (empty($worthstock_custom_meta['BrandName'])) {
        //     $worthstock_custom_meta['BrandName'] = get_bloginfo('name');
        // }

        $brands = $this->getProductCustomBrand($product_id);
        if (!$worthstock_custom_meta['BrandName'] && $brands) {
            $worthstock_custom_meta['BrandName'] = implode(' ', $brands);
            $worthstock_custom_meta['BrandName'] = trim($worthstock_custom_meta['BrandName']);
        }
        /* if ( empty(trim($worthstock_custom_meta['BrandName'])) ) {
            unset($worthstock_custom_meta['BrandName']);
        } */
            
        if ( is_array($worthstock_custom_meta) && count($worthstock_custom_meta) ) {
            $data = array_merge($data, $worthstock_custom_meta);
        }

        if ( !$data["IdProduct"] ) {
            unset($data["IdProduct"]);
        }
        
        if ($filter_type == "full") {
            if ( $product->is_type('variable') === false ) {
                $textual_attributes = $this->simpleProductAttributes($product, "textual");
                $textual_attributes = ( count($textual_attributes) ) ? implode(" - ", $textual_attributes) : "";
            }
            $data["Languages"][] = array(
                "Language" => $this->main()->default_language_code,
                "ProductName" => $product->get_name(),
                "Details" => $this->stripTagsShortcodes($product->get_description()),
                "Description" => $this->stripTagsShortcodes($product->get_short_description()),
                "Other" => $textual_attributes,
            );
            unset($data["Creator"]);
            // $data["Creator"] = "WOOCOMMERCE"; this field is ignored; default SHOPIFY

            $dimensions = $product->get_dimensions();

            if ( ! empty( $dimensions ) ) {
                // mandatory nella insert (e non nell'update)               
                $data["Weight"] = $data["Depth"] = $data["Width"] = $data["Height"] = "10";
                $data["Weight"] = $data["GrossWeight"] = "1000"; //1kg
                if ($product->get_weight())
                    $data["Weight"] = 1000 * $product->get_weight(); // grams to kilos
                if ($product->get_weight())
                    $data["GrossWeight"] = 1000 * $product->get_weight(); // grams to kilos
                if ($product->get_width())
                    $data["Width"] = $product->get_width();
                if ($product->get_height())
                    $data["Height"] = $product->get_height();
                if ($product->get_length())
                    $data["Depth"] = $product->get_length();
            }

        }

        //multiply variations computing all possible combinations and filling precalculated variations list
        if ( $product->is_type('variable')) {
            $this->woo_product_variations_elaborated = $this->preFilterVariationAttributes();
        }

        //$data["Variants"] = $product->get_children();
        if ( $product->is_type('variable') && $this->isVariableProductWithNonstandardAttributes($product) === false) {
            $data["Variants"] = $this->getProductVariationsFiltered($product, $filter_type);
        } else {
            //if not a variable product, then create a fake variation to be transmitted
            $data["Variants"] = $this->createFakeDefaultVariation($product, $filter_type);
        }

        
        $categories =  get_the_terms( $product_id, 'product_cat' );
        if ( $categories && ! is_wp_error( $categories ) ) {
            $main_product_category = $categories[0];
        }
        //$data["IdCategory"] = 1606233820;
        //$data["IdCategory"] = 12;
        /* if ( $cat_id = $this->main()->sync->getProdCatApiIdById($main_product_category->term_id) ) {
            $data["IdCategory"] = $cat_id;
        } */
        
        // tags solo per una insert
        if ($action_type == "insert") {
            $data["EnableShipping"] = 1;
            $data["FlagEcommerce"] = 1;
            $data["PickAtStore"] = 1;
            $data['Tags'] = $this->getProductCustomTaxonomyValue($product_id, 'product_tag');
            /* $tags = get_the_terms( $product_id, 'product_tag' );
            if (is_array($tags) && count($tags)) {
                $tags = wp_list_pluck($tags, 'name');
                $data['Tags'] = $tags;
            } */
            if (is_object($main_product_category)) {
                $data['Tags'][] = $main_product_category->slug;
            }
        }

        // se il prodotto è privato, il flag di visibilità su WS deve essere trasmesso come false
        if ( $product->get_status() == 'private' || !$product->is_visible() ) {
            $data["FlagEcommerce"] = 0;
        }

        
        if ( $product->is_type('variable') && $this->isVariableProductWithNonstandardAttributes($product) === true) {
            // here chain additional variants as products
            return $this->getProductVariationsAsExpandedProducts($product, $data, $filter_type);
        }
        
        return [$data];
    }

    /** return variation data filtered by filter_type */
    public function getProductVariationsFiltered($product, $filter_type) {
        if ( !$product->is_type('variable')) {
            return [];
        }
        //$variations = $product->get_available_variations();
        $variations = $this->woo_product_variations_elaborated;
        $filtered_data = [];
        
        //$variants_ids = $product->get_children();
        foreach ($variations as $idx => $variation) {
            $filtered_data[] = $this->mapVariationData($product, $variation, $filter_type, $idx);
        }
        return $filtered_data;

    }
    
    /**
     * utility function to compile a single variation json section
     */
    public function mapVariationData($product, $variation, $filter_type, $idx) {
        $variant_data = [];
        if ( $variation['variation_is_visible'] == false ) {
            return;
        }
        if ($this->default_variants) {
            $variant_data = $this->default_variants[0];
            unset($variant_data["IdProductVariant"]);
            unset($variant_data["DefaultVariant"]);
            unset($variant_data["Size"]);
            unset($variant_data["Color"]);
            unset($variant_data["Material"]);
            unset($variant_data["NA"]);
            unset($variant_data["Language"]);
        }
        
        if ($id_worthstock = $this->readWorthstockCustomParamValue($variation['variation_id'], 'IdProductVariant', "variant")) {
            $variant_data["IdProductVariant"] = $id_worthstock;
        }
        if ($filter_type == "quantity") {
            $variant_data["QuantityAvailable"] = (int) $variation['max_qty'];
        } else if ($filter_type == "price") {
            $variant_data["PriceLST"] = $variation['display_regular_price'];
            $variant_data["PriceB2C"] = $variation['display_price'];
            $variant_data["PricePROMO"] = $variation['sale_price'];
            $variant_data["PriceOVER"] = null;
        } else {
            if ($idx === 0) {
                $variant_data["DefaultVariant"] = 1;
            }
            //if ( isset($variation['image']['url']) && !empty($variation['image']['url']) ) {                
                $product_images = $this->getProductGalleryImages($product);
                array_unshift($product_images, $variation['image']['url']);
                $product_images = array_unique($product_images);
                $variant_data["Images"] = $product_images;
            //}
            if ($variation['is_in_stock']) {
                $variant_data["QuantityAvailable"] = (int) ($variation['max_qty'] >=1) ? $variation['max_qty'] : $this->min_unmanaged_stock;
            } else {
                $variant_data["QuantityAvailable"] = (int) 0;
            }
            $variant_data["PriceLST"] = $variation['display_regular_price'];
            $variant_data["PriceB2C"] = $variation['display_price'];
            $variant_data["PricePROMO"] = $variation['sale_price'];
            // mi serve un'identificativo univoco della variazione TODO: trovare modo migliore, perché vincolato alla presenza di uno sku per il prodotto
            /* 
            if ( !$variation['sku'] || $variation['sku'] == $product->get_sku()) {
                $variation['sku'] = $variation['sku'] . '-' . $variation['variation_id'];
            } */
            $variant_data["UniqueIdShopify"] = $this->generateUniqueIdShopify($product, $variation);
            // lo salvo per la variazione
            if ($variant_data["UniqueIdShopify"]) {
                update_post_meta( $variation['variation_id'], $this->getWorthstockCustomParamName('unique_id_shopify'), $variant_data["UniqueIdShopify"] );
            }
            $attributes = null;
            if (is_array($variation['attributes'])) {
                $attributes = $this->getVariationsAttributesList($variation);
                //$variant_data['Attributes'] = $attributes;
                /* 
                foreach ($attributes as $label => $value) {
                    $variant_data[$label] = $value;
                } */
                $variant_data["Languages"][] = $this->createLanguageSection($attributes, $this->main()->default_language_code);            
            } else {                
                $attributes = $this->getVariationsAttributesList(null);
                $variant_data["Languages"][] = $this->createLanguageSection($attributes, $this->main()->default_language_code);  
            }
            $variant_data["PriceOVER"] = null;
        }
        return $variant_data;

    }

    /**
     * just arrange data from the main product as a variation, since worthstock expects it
     */
    public function createFakeDefaultVariation($product, $filter_type) {
        $variant_data = [];
        $variant_data["DefaultVariant"] = 1;
        $product_id = $product->get_id();
        
        // leggiamo id_variation dal prodotto, visto che la variazione non esiste
        //if ($id_worthstock = $this->readWorthstockCustomParamValue($variation['variation_id'], 'IdProductVariant', "variant")) {
        if ($id_variation = get_post_meta($product_id, $this->getWorthstockCustomParamName('id_product_variant'), true)) {
            $variant_data["IdProductVariant"] = $id_variation;
        }
        
        if ($filter_type == "quantity") {
            $variant_data["QuantityAvailable"] = $this->productGetStockQuantity($product);
        } else if ($filter_type == "price") {
            $variant_data["PriceLST"] = $product->get_regular_price();
            $variant_data["PriceB2C"] = $product->get_price();
            $variant_data["PricePROMO"] = $product->get_sale_price();
            $variant_data["PriceOVER"] = null;
        } else {
            //if ( isset($product['image']['url']) && !empty($product['image']['url']) ) {
                // $image_id  = $product->get_image_id();
                // $image_url = wp_get_attachment_image_url( $image_id, 'full' );
                // $variant_data["Images"] = array($image_url);
                $variant_data["Images"] = $this->getProductGalleryImages($product);

            //}
            $variant_data["QuantityAvailable"] = $this->productGetStockQuantity($product);
            $variant_data["PriceLST"] = $product->get_regular_price();
            $variant_data["PriceB2C"] = $product->get_price();
            $variant_data["PricePROMO"] = $product->get_sale_price();
            $variant_data["PriceOVER"] = null;
            $variant_data["UniqueIdShopify"] = $this->generateUniqueIdShopify($product);
            $attributes = null;            
            $attributes = $this->simpleProductAttributes($product, "array");
            //if ( $attributes = $product->get_attributes() && is_array($attributes) ) {
            if ( is_array($attributes) && count($attributes) ) {
                $product_attributes = $this->getVariationsAttributesList(['attributes'=>$attributes]);
                $variant_data["Languages"][] = $this->createLanguageSection($product_attributes, $this->main()->default_language_code);
                //$variant_data['Attributes'] = $attributes;
                /* foreach ($attributes as $label => $value) {
                    $variant_data[$label] = $value;
                } */

            } else {               
                $product_attributes = $this->getVariationsAttributesList(null);
                $variant_data["Languages"][] = $this->createLanguageSection($product_attributes, $this->main()->default_language_code);
            }
        }
        return [$variant_data];
    }

    /**
     * extract WS attributes for the provided variation
     */
    protected function getVariationsAttributesList($variation) {
        /* if ( is_int($variation) && !$variation['variation_id'] ) {
            //$variation = wc_get_product($variation);
            return [];
        } */

        //foreach( $variation->get_attributes() as $taxonomy => $term_slug )
        // Attributes
        $attributes = array();
        foreach ($this->default_attributes as $default) {
            $attributes[$default] = null;            
        }
        foreach( $this->filterVariationAttributesList($variation['attributes']) as $taxonomy_label => $term_name ) {
            if ( !($translated_attribute = $this->translateDefaultAttribute($taxonomy_label)) ) {
                continue;
            }
            $attributes[$translated_attribute] = $term_name;
        }
        if ( !array_filter($attributes) ) {
        //if ( empty($attributes) ) {
            $attributes["NA"] = "N.A.";
        }
        return $attributes;
    }

    /**
     * Given a variation filter the correct pair of Attribute name and Attribute value (taking in account simple attributes and taxonomy attributes)
     */
    protected function filterVariationAttributesList($attributes) {
        $elaborated_attributes = array();
        if (is_array($attributes)) {
            foreach( $attributes as $attribute => $value ) {
                if (!$value) {
                    unset($attributes[$attribute]);
                    continue;
                }
                $taxonomy = str_replace('attribute_', '', $attribute );
                //if ( $value->is_taxonomy() ) {
                $taxonomy_label = get_taxonomy( $taxonomy )->labels->singular_name;
                if ($taxonomy_label) {
                    $term_name = get_term_by( 'slug', $value, $taxonomy )->name;
                } else {
                    $taxonomy_label =  ucfirst(str_replace('pa_', '', $taxonomy ));
                    $term_name = $value;
                }
                $elaborated_attributes[$taxonomy_label] = $term_name;
            }
        }
        return $elaborated_attributes;
    }

    /**
     * sort out attributes for variation, taking in account default attributes
     */
    protected function preFilterVariationAttributes() {
        $product_attributes = $this->woo_product->get_variation_attributes();
        $initial_variations = $this->woo_product->get_available_variations();
        foreach( $product_attributes as $attribute => $values ) {
            $next_variations = ( ( isset($attribute_variations) && !empty($attribute_variations) ) ? $attribute_variations : $initial_variations);
            $attribute_variations = $this->multiplexVariationByAttributes($next_variations, $attribute, $values);
            //$elaborated_variations = array_merge($elaborated_variations, $attribute_variations);
        }
        return $attribute_variations;
    }

    /**
     * take an attribute and multiply the values with the variations
     */
    protected function multiplexVariationByAttributes($variations, $attribute, $values) {
        $attribute_variations = array();
        foreach ($variations as $idx => $variation) {
            // if the variation hasn't default values for the attribute, all possible values are considered
            if ( empty($variation['attributes']['attribute_'.$attribute]) && array_key_exists('attribute_'.$attribute, $variation['attributes']) && is_array($values) ) {                    
                foreach($values as $i=>$v) {
                    if ( $i >= 1) {
                        unset($variation["IdProductVariant"]);
                        unset($variation["variation_id"]);
                    }
                    $variation['attributes']['attribute_'.$attribute] = $v;
                    $attribute_variations[] = $variation;
                }
            } else {
                $attribute_variations[] = $variation;
            }
        }
        return $attribute_variations;
    }

    /**
     * extract main product attributes (not from variations)
     */
    public function simpleProductAttributes($product, $output="textual") {        
        $attributes = array_filter( $product->get_attributes(), 'wc_attributes_array_filter_visible' );
        $textual_attributes = array();
        foreach ( $attributes as $attribute ) {
            $values = array();

            if ( $attribute->is_taxonomy() ) {
                $attribute_taxonomy = $attribute->get_taxonomy_object();
                $attribute_values   = wc_get_product_terms( $product->get_id(), $attribute->get_name(), array( 'fields' => 'all' ) );

                foreach ( $attribute_values as $attribute_value ) {
                    $value_name = esc_html( $attribute_value->name );
                    $values[] = $value_name;
                }
            } else {
                $values = $attribute->get_options();
                foreach ( $values as &$value ) {
                    $value = esc_html( $value );
                }
            }
            if ($output == "textual") {
                if ($this->translateDefaultAttribute(wc_attribute_label( $attribute->get_name() )) && count($values) == 1) {
                    continue;
                }
                $textual_attributes[] = wc_attribute_label( $attribute->get_name() ) . ": " . apply_filters( 'woocommerce_attribute', wptexturize( implode( ', ', $values ) ), $attribute, $values );
            } else {
                if ($this->translateDefaultAttribute(wc_attribute_label( $attribute->get_name() )) && count($values) > 1) {
                    continue;
                }
                $textual_attributes[$attribute->get_name()] = $values[0];
            }

            /* $product_attributes[ 'attribute_' . sanitize_title_with_dashes( $attribute->get_name() ) ] = array(
                'label' => wc_attribute_label( $attribute->get_name() ),
                'value' => apply_filters( 'woocommerce_attribute', wpautop( wptexturize( implode( ', ', $values ) ) ), $attribute, $values ),
            ); */
        }
        //return ( count($textual_attributes) ) ? implode(" - ", $textual_attributes) : "";
        return $textual_attributes;
    }

    public function getProductGalleryImages($product) {
        $imagesArray = [];
        if ($product->get_image_id()) {
            $imagesArray[] = wp_get_attachment_image_url( $product->get_image_id(), 'full' );
        }
        $images_ids = $product-> get_gallery_image_ids();
        for($i = 0, $j = count($images_ids); $i < $j;$i++ ){
            $imagesArray[] = wp_get_attachment_image_url($images_ids[$i], 'full');
        }
        //$product->gallery = $images_arr;
        //array_push($imagesArray, $product);
        return array_unique($imagesArray);
    }

    public function createLanguageSection($data, $language="IT") {
        $language_section["Language"] = $language;
        if (is_array($data)) {
            $language_section = array_merge($language_section, $data);
        }
        return $language_section;
    }

    /**
     * whether the product has stock management activated
     */
    public function hasProductStockManagement($product) {
        return $product->managing_stock();
    }

    /**
     * if variable product has attributes outside size, material, color
     */
    public function isVariableProductWithNonstandardAttributes($product) {        
        //$productHasStandardAttribute = true;
        //$variations = $product->get_available_variations();
        $variations = $this->woo_product_variations_elaborated;
        foreach ($variations as $idx => $variation) {
            
            foreach( $this->filterVariationAttributesList($variation['attributes']) as $taxonomy_label => $term_name ) {
            //if (is_array($variation['attributes'])) {
            //    foreach( $variation['attributes'] as $key => $value ) {
                if (!$term_name) continue;
                if ( !($translated_attribute = $this->translateDefaultAttribute($taxonomy_label)) ) {
                    return true;
                }
            //}
            }
        }
        return false;
    }

    /**
     * sometimes WOO products (and variations) with custom attributes map to multiple WS products (and variations)
     */
    public function getProductVariationsAsExpandedProducts($product, $template_data, $filter_type) {
        $products = [];
        
        $grouped_products = $this->getVirtualWorthstockProducts($product, $template_data, $filter_type);
        
        $template_data["Variants"] = null;
        $include_id_product = true;
        
        $IdProducts = get_post_meta($product->get_id(), $this->main()->mapper->getWorthstockCustomParamName('id_product'));

        if ( count($IdProducts) <> count($grouped_products) ) {
            $include_id_product = false;
        }
        
        $prog = 0;
        foreach ($grouped_products as $variations) {
            $product_data = $template_data;
            $prog++;

            $IdProduct = current($IdProducts);
            next($IdProducts);
            
            if ($IdProduct && $include_id_product) {
                $product_data["IdProduct"] = $IdProduct;
            } else {
                unset($product_data["IdProduct"]);
            }
            $product_data["Languages"][0]["ProductName"] = $this->getProductNameByAttributes($variations[0], $template_data["Languages"][0]["ProductName"]);
            
            foreach ($variations as $idx => $variation) {
                
                $product_data["Variants"][] = $filtered_data[] = $this->mapVariationData($product, $variation, $filter_type, $idx);
                // se l'IdProduct non corrisponde a quello salvato nella variazione, non lo passo (variazione ricreata)
                $id_product_variation = $this->readWorthstockCustomParamValue($variation['variation_id'], 'IdProduct', 'variant');
                if ($IdProduct != $id_product_variation) {
                    unset($product_data["Variants"][$idx]["IdProductVariant"]);
                }
                
                /* if ($id_worthstock = $this->readWorthstockCustomParamValue($variation['variation_id'], 'IdProduct', 'variant')) {
                    $product_data["IdProduct"] = $id_worthstock;
                } else {
                    //continue;
                    unset($product_data["IdProduct"]);
                } */

            }
            $products[] = $product_data;
        }
        return $products;
    }

    
    /**
     * compute WS map corresponsing to a WOO product
     */
    public function getVirtualWorthstockProducts($product, $template_data, $filter_type) {
        $grouped_products = [];
        //$variations = $product->get_available_variations();
        $variations = $this->woo_product_variations_elaborated;        
        foreach ($variations as $idx => $variation) {
            $product_data = $template_data;
            if ($id_worthstock = $this->readWorthstockCustomParamValue($variation['variation_id'], 'IdProductVariant', 'variant')) {
            #if ($id_worthstock = get_post_meta($variation['variation_id'], '_wst_id_product_variant', true) ) {
                $product_data["IdProduct"] = $id_worthstock;
            } else {
                unset($product_data["IdProduct"]);
            }
            $worthstock_title = $this->getProductNameByAttributes($variation, $template_data["Languages"][0]["ProductName"]);

            $grouped_products[md5($worthstock_title)][] = $variation;
        }
        return $grouped_products;
    }

    /**
     * if product has no stock management, then stock quantity returned is 1
     */
    public function productGetStockQuantity($product) {
        // default, not in stock
        $quantity = 0;
        // if managed, return quantity
        if ($product->managing_stock()) {
            $quantity = $product->get_stock_quantity();
        // else, if in stock return 1
        } else if ($product->is_in_stock() || $product->is_on_backorder()) {
            $quantity = $this->min_unmanaged_stock;
        }
        return $quantity;
    }

    /**
     * combine product name + custom attributes
     */
    public function getProductNameByAttributes($variation, $basetitle) {
        if ( is_int($variation) && !$variation['variation_id'] ) {
            return $basetitle;
        }
        // Attributes
        $terms_string = null;        
        foreach( $this->filterVariationAttributesList($variation['attributes']) as $taxonomy_label => $term_name ) {
        //foreach( $variation['attributes'] as $attribute => $value ) {
            if (!$term_name) continue;
            if ( ($this->translateDefaultAttribute($taxonomy_label)) ) {
                continue;
            }
            $terms_string .= " " . $taxonomy_label . " " . $term_name;
        }
        return trim($basetitle . ($terms_string ? " -" . $terms_string : ''));
    }

    public function generateUniqueIdShopify($product, $variation=null) {
        $UniqueIdShopify = $product->get_id();
        if (isset($variation) && $variation['variation_id']) {
            $UniqueIdShopify .= '_' . $variation['variation_id'];
        }
        return $UniqueIdShopify;
    }

    public function isProductSynced($product) {        
        return ($id_worthstock = $this->readWorthstockCustomParamValue($product->ID, 'IdProduct')) ? $id_worthstock : false;
    }

    /**
     * conversion list of worthstock fields to woo custom meta
     */
    public static function getWorthstockCustomParamList($obj="product") {
        $new_values = [];        
        $params = self::getWorthstockFullCustomParamList($obj);
        
        foreach($params as $key => $values) {
            $new_values[$key] = $values["name"];
        }

        return $new_values;
    }
    
    /**
     * conversion list of worthstock fields to woo custom meta
     */
    public static function getWorthstockFullCustomParamList($obj="product") {
        if ($obj == "product") {
            return array(
                "IdProduct"=>["name"=>"id_product","fullname"=>__("IdProduct","worthstock"),"type"=>"int","readonly"=>true],
                //"IdProductVariant"=>["name"=>"id_product_variant","fullname"=>__("IdProductVariant","worthstock"),"type"=>"int","readonly"=>true],
                "Barcode"=>["name"=>"barcode","fullname"=>__("Barcode","worthstock"),"type"=>"string"],
                "BrandName"=>["name"=>"brand_name","fullname"=>__("Brand Name","worthstock"),"type"=>"string"],
                "Creator"=>["name"=>"creator","fullname"=>__("Creator","worthstock"),"type"=>"string","readonly"=>true],
                //"ShippingFree"=>["name"=>"shipping_free","fullname"=>__("Spedizione gratuita con WORTHSTOCK","worthstock"),"type"=>"bool"],
                "FlagEcommerce"=>["name"=>"flag_ecommerce","fullname"=>__("Visibile in E-Commerce","worthstock"),"type"=>"bool","selected"=>true],
                "EnableShipping"=>["name"=>"enable_shipping","fullname"=>__("Abilita le spedizioni","worthstock"),"type"=>"bool","selected"=>true],
                "PickAtStore"=>["name"=>"pick_at_store","fullname"=>__("Ritiro presso il Punto Vendita","worthstock"),"type"=>"bool","selected"=>true],
                "ReturnAlwaysFree"=>["name"=>"return_always_free","fullname"=>__("Reso gratuito con WORTHSTOCK","worthstock"),"type"=>"bool"],
                "MadeInItaly"=>["name"=>"made_in_italy","fullname"=>__("Made in Italy","worthstock"),"type"=>"bool"],
                "ShopCourier"=>["name"=>"shop_courier","fullname"=>__("Spedizione a carico del Venditore","worthstock"),"type"=>"bool"]                
            );
        } else {
            return array(
                "IdProduct"=>["name"=>"id_product","type"=>"int","readonly"=>true],
                "IdProductVariant"=>["name"=>"id_product_variant","type"=>"int","readonly"=>true],
                "UniqueIdShopify"=>["name"=>"unique_id_shopify","type"=>"int","readonly"=>true]
            );
        }
    }

    public static function getWorthstockCustomParamName($param_name) {
        return "_wst_" . $param_name;
    }
    
    public function readWorthstockCustomParamValue($product_id, $ws_param_name, $obj="product", $check_existence=true) {
        $params = $this->getWorthstockFullCustomParamList($obj);
        if ($params[$ws_param_name]) {
            $param_name = $this->getWorthstockCustomParamName($params[$ws_param_name]["name"]);
            /* $old_param_name =  "_" . $params[$ws_param_name]["name"];
            $old_value = get_post_meta($product_id, $old_param_name, true);
            if ($old_value) {
                $status = update_post_meta( $product_id, $param_name, $old_value );
                if ($status) {
                    delete_post_meta($product_id, $old_param_name);
                }
                return ($params[$ws_param_name]["type"] == "bool") ? (int) $old_value : $old_value;
            } */
            if ( $check_existence === false || metadata_exists( 'post', $product_id, $param_name ) !== false ) {
                $value = get_post_meta($product_id, $param_name, true);
            } else {
                return false;
            }
            return ($params[$ws_param_name]["type"] == "bool") ? (int) $value : $value;
        }
        return false;
    }
    
    public function readWorthstockCustomAllValues($product_id) {        
        $product_meta = [];
        $params = $this->getWorthstockFullCustomParamList();
        foreach($params as $key => $value) {
            $product_meta[$key] = $this->readWorthstockCustomParamValue($product_id, $key, "product", false);
        }
        return $product_meta;
    }

    public function getProductCustomTaxonomyValue($product_id, $taxonomy) {
        $terms = get_the_terms( $product_id, $taxonomy );
        if ($terms) {
            $terms = wp_list_pluck($terms, 'name');
        }
        return $terms;
    }

    public function getProductCustomBrand($product_id) {
        $terms = $this->getProductCustomTaxonomyValue( $product_id, 'product_brand' );
        if ($terms) {
            return $terms;
        }
        $terms = $this->getProductCustomTaxonomyValue( $product_id, 'product_brands' );
        if ($terms) {
            return $terms;
        }
        $terms = $this->getProductCustomTaxonomyValue( $product_id, 'brands' );
        if ($terms) {
            return $terms;
        }
        $terms = $this->getProductCustomTaxonomyValue( $product_id, 'brand' );
        if ($terms) {
            return $terms;
        }
        $terms = $this->getProductCustomTaxonomyValue( $product_id, 'brandname' );
        if ($terms) {
            return $terms;
        }
        return false;
    }

    public function stripTagsShortcodes($content) {
        $content = strip_tags($content);
        $content = strip_shortcodes($content);
        //$content = preg_replace('#\[[^\]]+\]#', '', $content);
        return $content;
    }

    public function getWSProductStats($data) {
        $stats = array('ProductsNumber'=>0, 'VariantsNumber'=>0);
        if (is_array($data)) {
            $stats['ProductsNumber'] = count($data);
            $variants_number = 0;
            foreach($data as $p) {
                if (isset($p['Variants'])) {
                    $variants_number += count($p['Variants']);
                }    
            }
            $stats['VariantsNumber'] = $variants_number;
        }
        return $stats;
    }

}